/**
 * Copyright 2016 - present, Matthieu Jabbour <matthieu.jabbour@gmail.com>.
 * All rights reserved.
 */


/* eslint-disable no-console */


process.env.NODE_ENV = 'production';


const fs = require('fs-extra');
const webpack = require('webpack');
const config = require('../config/webpack.config.prod');


const compiler = webpack(config);
const distPath = config.output.path;


// Removing existing `dist` directory...
fs.remove(distPath)
  // Running webpack compiler...
  .then(() => new Promise((resolve, reject) => {
    console.log('Compiling...');
    compiler.run((error, stats) => (error ? reject(error) : resolve(stats)));
  }))
  // Displaying webpack compilation stats...
  .then((stats) => {
    if (stats.hasErrors()) throw new Error(stats.toJson().errors[0]);
    console.log(stats.toString({
      colors: true,
      cached: true,
      performance: true,
      modules: false,
      children: false,
    }));
  })
  // All went well...
  .then(() => {
    console.log('Done.');
  })
  // If any error occurs...
  .catch((error) => {
    console.error(error);
  });
