/**
 * Copyright 2016 - present, Matthieu Jabbour <matthieu.jabbour@gmail.com>.
 * All rights reserved.
 */


/* eslint-disable no-console */


process.env.NODE_ENV = 'development';


const fs = require('fs-extra');
const path = require('path');
const express = require('express');
const webpack = require('webpack');
const webpackDevMiddleware = require('webpack-dev-middleware');
const webpackHotMiddleware = require('webpack-hot-middleware');
const config = require('../config/webpack.config.dev');


const server = express();
const compiler = webpack(config);
const distPath = config.output.path;


// Removing existing `dist` directory...
fs.remove(distPath)
  // Running webpack compiler...
  .then(() => {
    // `webpack-dev-middleware` package enables Webpack to be run as a server in watch mode,
    // for a better development experience.
    // We use it instead of `webpack-dev-server` package because it offers more flexibility and
    // control over the server.
    server.use(webpackDevMiddleware(compiler, {
      stats: {
        colors: true,
        cached: true,
        performance: true,
        modules: false,
        children: false,
      },
      publicPath: '/assets',
      index: 'index.html',
    }));

    // `webpack-hot-middleware` package enables HMR, along with a dev server.
    server.use(webpackHotMiddleware(compiler, {
      path: '/assets/_hmr',
      heartbeat: 2000,
    }));

    server.use(express.static(path.resolve(config.output.path, '..')));
    server.listen(3000, () => {
      console.log('Starting dev server...');
    });
  })
  // If any error occurs...
  .catch((error) => {
    console.error(error);
  });
